/*
* JBoss, the OpenSource EJB server
*
* Distributable under LGPL license.
* See terms of license at gnu.org.
*/
package org.jboss.docs.jms.client;

import java.io.InputStream;

import java.net.URL;
import java.net.HttpURLConnection;

import javax.management.MBeanServerFactory;
import javax.management.MBeanServer;
import javax.management.ObjectName;

import javax.jms.Topic;
import javax.jms.Queue;

/**
 * Demonstration of how to administer JMS destinations programatically for 
 * JBossMQ for 2.5.x. Remember that destinations are not persisted between server reboots.
 *
 * To use from a remote client, adapt the HOST and PORT variable to your local
 * settings.
 *
 * Only works up to and including JBoss 2.4.x
 *
 * @author Peter Antman
 * @version $Revision: 1.1 $
 */

public class DestinationHelper25  {
    /**
     * Host where JBoss is
     */
    static final String HOST = "localhost";

    /**
     * Port of the JMX HTML admin gui
     */
    static final int PORT = 8082;


    /**
     * Base url to invoke, MBean PATH has changed in 2.5.x
     */
    static final String BASE_URL_ARG = "/InvokeAction//JBossMQ%3Aservice%3DServer"; 


    /**
     * Create a destination. Works both if you are in the same VM as JBossMQ
     * or if you are a remote client. New method names in 2.5.x.
     *
     * @param type Topic.class or Queue.class
     * @param name name of the destination, without prefix queue/ or topic/
     */
    public static void createDestination(Class type, String name) 
	throws Exception
    {
	String method = null;
	if (type == Topic.class) {
	    method = "createTopic";
	}
	else if (type == Queue.class) {
	    method = "createQueue";
	}
	invoke(method, name);
      
    }
    
    /**
     * Destroy a destination. Works both if you are in the same VM as JBossMQ
     * or if you are a remote client.
     *
     * @param type Topic.class or Queue.class
     * @param name name of the destination, without prefix queue/ or topic/
     */
    public static void destroyDestination(Class type, String name) 
	throws Exception
    {
	String method = null;
	if (type == Topic.class) {
	    method = "destroyTopic";
	}
	else if (type == Queue.class) {
	    method = "destroyQueue";
	}
	invoke(method, name);
    }
    
    /**
     * Helper method, decides if we are in the same VM as JBossMQ, or are a
     * remote client.
     *
     * @param method name of the MBean method to invoke
     * @param destName name of the destination
     */
    protected static void invoke(String method, String destName) 
	throws Exception
    {
	// Check if we can get the MBean server
	try {
	    MBeanServer server = (MBeanServer)
		MBeanServerFactory.findMBeanServer(null).iterator().next();
	 
	    // We are native to the MBean server, invoke directly
	    invokeViaMBean(method, destName);
	}catch(Exception ex) {
	  
	    // We are a standalone client, invoke via URL
	    invokeViaUrl(method, destName);
	}
      
    }

    /**
     * Invoke the JMSServer Mbean via URL.
     *
     * @param method name of the MBean method to invoke
     * @param destName name of the destination
     */
    protected static void invokeViaUrl(String method, String destName) 
	throws Exception
    {
	// Construct the correct url, see to it that HOST and PORT are correct
	String action = "action=" + method + "?action=" + method + 
	    "&param0%2Bjava.lang.String=" + destName;

	String arg = BASE_URL_ARG + "/" + action;
      
	URL url = new URL("http", HOST, PORT, arg);

	//Invoke the URL

	HttpURLConnection con = (HttpURLConnection)url.openConnection();
	con.connect();

	// Get the content
	InputStream is = con.getInputStream();
	java.io.ByteArrayOutputStream os = new  java.io.ByteArrayOutputStream();
	byte[] buff = new byte[1024];
	for(;;) {
	    int size = is.read( buff );
	    if (size == -1 ) {
		break;
	    }
	    os.write(buff, 0, size);
	}
	os.flush();
      
	// The response codes from JMX HTTP does not seem to work.
	if (con.getResponseCode() != HttpURLConnection.HTTP_OK ) {
	    throw new Exception ("Could not invoke url: " + con.getResponseMessage() );
	} else {
	    System.out.println("Invoked URL: " + method + " for destination " + destName + "got resonse: " + os.toString());
	}
      
      
    }

    /**
     * Invoke the JMSServer Mbean via the MBean server.
     *
     * @param method name of the MBean method to invoke
     * @param destName name of the destination
     */
    protected static void invokeViaMBean(String method, String destName) 
	throws Exception
    {
	// Get the MBean server
	MBeanServer server = (MBeanServer)
	    MBeanServerFactory.findMBeanServer(null).iterator().next();
      
	// Invoke the MBean
	server.invoke(new ObjectName("JBossMQ", "service", "Server"),
		      method,
		      new Object[] { destName },
		      new String[] { "java.lang.String" });
    }


   
    /**
     * Simple test. Creates topic destination myCreated if invoked without argument, and destroys myCreated if invoked with any argument.
     */
    public static void main(String[] args) {
	try {
	    if (args.length >0){
		destroyDestination(Topic.class,"myCreated");
	    }else {
		createDestination(Topic.class,"myCreated");
	    }


	}catch(Exception ex) {
	    System.out.println("Error in administering destination: " + ex);
	    ex.printStackTrace();
	}
    }
   
} // DestinationHelper
