/*
* JBoss, the OpenSource EJB server
*
* Distributable under LGPL license.
* See terms of license at gnu.org.
*/

package org.jboss.docs.jms.client;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import javax.jms.TopicConnectionFactory;
import javax.jms.TopicConnection;
import javax.jms.TopicSession;
import javax.jms.TopicPublisher;
import javax.jms.Topic;
import javax.jms.TextMessage;
import javax.jms.Session;
import javax.jms.JMSException;
/**
 * <p>Simple JMS client, publish text messages to testTopic Topic.
 * </p>
 *
 * <p><b>NOTE</b>This code is showcase only. It may not provide a stable production example.</p>
 * @author Peter Antman
 * @version $Revision: 1.1 $
 */

public class HelloPublisher {

   /** 
    * Topic connection, hold on to this so you may close it. 
    */
   TopicConnection topicConnection;
   
   /** 
    * Topic session, hold on to this so you may close it. Also used to create messages. 
    */
   TopicSession topicSession;

   /** 
    * Use this to publish messages.
    */
   TopicPublisher topicPublisher;

   /** 
    * Destination to publish to 
    */
   Topic topic;
   

   /**
    * Sets up all the JMS fixtures.
    *
    * Use close() when finished with object.
    *
    * @param factoryJNDI name of the topic connection factory to look up.
    * @param topicJNDI name of the topic destination to look up
    */
   public HelloPublisher(String factoryJNDI, String topicJNDI) throws JMSException, NamingException {
      // Get the initial context
      Context context = new InitialContext();

      // Get the connection factory
      TopicConnectionFactory topicFactory = 
		(TopicConnectionFactory)context.lookup(factoryJNDI);
      
      // Create the connection 
      topicConnection = topicFactory.createTopicConnection();

      // Create the session
      topicSession = topicConnection.createTopicSession(
						   // No transaction
						   false, 
						   // Auto ack
						   Session.AUTO_ACKNOWLEDGE);
      
      // Look up the destination
      topic = (Topic)context.lookup(topicJNDI);
      
      // Create a publisher
      topicPublisher = topicSession.createPublisher(topic);
  
   }

   /**
    * Publish the given String as a JMS message to the testTopic topic.
    */
   public void publish(String msg) throws JMSException {

      // Create a message
      TextMessage message = topicSession.createTextMessage();
      message.setText(msg);

      // Publish the message
      topicPublisher.publish(topic, message);
   }

   /**
    * Close session and connection. When done, no publishing is possible any more.
    */
   public void close() throws JMSException {
      topicSession.close();
      topicConnection.close();
   }
   
   /**
    * Run an example publishing 10 messages to testTopic. Only works up to and including JBoss 2.4.x
    */
   public static void main(String[] args) {
      try {
	 
	 //Create the HelloPublisher, giving it the name of the TopicConnection
	 //Factory and the Topic destination to use in lookup.
	 
	 HelloPublisher publisher = new HelloPublisher(
	    // Name of ConnectionFactory
	    "TopicConnectionFactory",
	    // Name of destination to publish to
	    "topic/testTopic"
	    );
	 
	 // Publish 10 messages
	 for (int i = 1; i < 11; i++) {
	    String msg = "Hello World no. " + i;
	    System.out.println("Publishing message: " + msg);
	    publisher.publish(msg);
	 }
	 
	 // Close down your publisher
	 publisher.close();
	 
      }catch(Exception ex) {
	 System.err.println("An exception occured while testing HelloPublisher: " + ex);
	 ex.printStackTrace();
      }
      
   }
   
} // HelloPublisher








