/*
* JBoss, the OpenSource EJB server
*
* Distributable under LGPL license.
* See terms of license at gnu.org.
*/

package org.jboss.docs.jms.client;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import javax.jms.QueueConnectionFactory;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.QueueReceiver;
import javax.jms.Queue;
import javax.jms.Message;
import javax.jms.TextMessage;
import javax.jms.Session;
import javax.jms.MessageListener;
import javax.jms.JMSException;

/**
 * <p>Simple JMS client, subsribes for messages on testQueue Queue.
 * </p>
 *
 * <p><b>NOTE</b>This code is showcase only. It may not provide a stable production example.</p>
 *
 * @author Peter Antman
 * @version $Revision: 1.1 $

 */

public class HelloReceiver implements MessageListener {
   
   /** 
    * Queue connection, hold on to this so you may close it. 
    */
   QueueConnection queueConnection;
   
   /** 
    * Queue session, hold on to this so you may close it. Also used to create messages. 
    */
   QueueSession queueSession;

   /** 
    * Subsrciber
    */
   QueueReceiver queueReceiver;

   /** 
    * Destination to subscribe to 
    */
   Queue queue;

   /**
    * Sets up all the JMS fixtures.
    *
    * Use close() when finished with object.
    *
    * @param factoryJNDI name of the queue connection factory to look up.
    * @param queueJNDI name of the queue destination to look up
    */
   public HelloReceiver(String factoryJNDI, String queueJNDI) throws JMSException, NamingException 
   {
      // Get the initial context
      Context context = new InitialContext();

      // Get the connection factory
      QueueConnectionFactory queueFactory = 
	 (QueueConnectionFactory)context.lookup(factoryJNDI);
      
      // Create the connection 
      queueConnection = queueFactory.createQueueConnection();

      // Create the session
      queueSession = queueConnection.createQueueSession(
	 // No transaction
	 false, 
	 // Auto ack
	 Session.AUTO_ACKNOWLEDGE);
      
      // Look up the destination
      queue = (Queue)context.lookup(queueJNDI);
      
      // Create a subsriber
      queueReceiver = queueSession.createReceiver(queue);

      // Set the message listener, which is this class since we implement
      // the MessageListener interface
      queueReceiver.setMessageListener(this);
	
      System.out.println("HelloReceiver subscribed to queue: " + queueJNDI);
      // OBS! For the message listener to receive any messages the connection
      // has to be started.
      queueConnection.start();
   }
    
   /**
    * Implementation of the MessageListener interface, messages will be received through this method.
    */
   public void onMessage(Message m) {
      // Unpack the message, be carefull when casting to correct message type
      // onMessage should not through any application exceptions.
      try {
	 String msg = ((TextMessage)m).getText();
	 System.out.println("HelloReceiver got message: " + msg);
      }catch(JMSException ex) {
	 System.err.println("Could not get text message: " + ex);
	 ex.printStackTrace();
      }
   }
    
   /**
    * Close session and connection.
    */
   public void close() throws JMSException {
      queueSession.close();
      queueConnection.close();
   }
   
   /**
    * Run an example subscribing to queue testQueue. Only works up to and including JBoss 2.4.x
    */
   public static void main(String[] args) {
      try {
	 
	 //Create the HelloSubsriber, giving it the name of the QueueConnection
	 //Factory and the Queue destination to use in lookup.
	 
	 HelloReceiver receiver = new HelloReceiver(
	    // Name of ConnectionFactory
	    "QueueConnectionFactory",
	    // Name of destination to publish to
	    "queue/testQueue"
	    );
	 
      }catch(Exception ex) {
	 System.err.println("An exception occured while testing HelloReceiver: " + ex);
	 ex.printStackTrace();
      }
   }
   
} // HelloReceiver
