/*
* JBoss, the OpenSource EJB server
*
* Distributable under LGPL license.
* See terms of license at gnu.org.
*/

package org.jboss.docs.jms.client;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import javax.jms.QueueConnectionFactory;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.QueueSender;
import javax.jms.Queue;
import javax.jms.TextMessage;
import javax.jms.Session;
import javax.jms.JMSException;
/**
 * <p>Simple JMS client, send text messages to testQueue Queue.
 * </p>
 *
 * <p><b>NOTE</b>This code is showcase only. It may not provide a stable production example.</p>
 * @author Peter Antman
 * @version $Revision: 1.1 $
 */

public class HelloSender {

   /** 
    * Queue connection, hold on to this so you may close it. 
    */
   QueueConnection queueConnection;
   
   /** 
    * Queue session, hold on to this so you may close it. Also used to create messages. 
    */
   QueueSession queueSession;

   /** 
    * Use this to send messages.
    */
   QueueSender queueSender;

   /** 
    * Destination to send to 
    */
   Queue queue;
   

   /**
    * Sets up all the JMS fixtures.
    *
    * Use close() when finished with object.
    *
    * @param factoryJNDI name of the queue connection factory to look up.
    * @param queueJNDI name of the queue destination to look up
    */
   public HelloSender(String factoryJNDI, String queueJNDI) throws JMSException, NamingException {
      // Get the initial context
      Context context = new InitialContext();

      // Get the connection factory
      QueueConnectionFactory queueFactory = 
		(QueueConnectionFactory)context.lookup(factoryJNDI);
      
      // Create the connection 
      queueConnection = queueFactory.createQueueConnection();

      // Create the session
      queueSession = queueConnection.createQueueSession(
						   // No transaction
						   false, 
						   // Auto ack
						   Session.AUTO_ACKNOWLEDGE);
      
      // Look up the destination
      queue = (Queue)context.lookup(queueJNDI);
      
      // Create a sender
      queueSender = queueSession.createSender(queue);
  
   }

   /**
    * Send the given String as a JMS message to the testQueue queue.
    */
   public void send(String msg) throws JMSException {

      // Create a message
      TextMessage message = queueSession.createTextMessage();
      message.setText(msg);

      // Send the message
      queueSender.send(queue, message);
   }

   /**
    * Close session and connection. When done, no sending is possible any more.
    */
   public void close() throws JMSException {
      queueSession.close();
      queueConnection.close();
   }
   
   /**
    * Run an example sending 10 messages to testQueue. Only works up to and including JBoss 2.4.x
    */
   public static void main(String[] args) {
      try {
	 
	 //Create the HelloSender, giving it the name of the QueueConnection
	 //Factory and the Queue destination to use in lookup.
	 
	 HelloSender sender = new HelloSender(
	    // Name of ConnectionFactory
	    "QueueConnectionFactory",
	    // Name of destination to send to
	    "queue/testQueue"
	    );
	 
	 // Send 10 messages
	 for (int i = 1; i < 11; i++) {
	    String msg = "Hello World no. " + i;
	    System.out.println("Sending message: " + msg);
	    sender.send(msg);
	 }
	 
	 // Close down your sender
	 sender.close();
	 
      }catch(Exception ex) {
	 System.err.println("An exception occured while testing HelloSender: " + ex);
	 ex.printStackTrace();
      }
      
   }
   
} // HelloSender








